#![feature(specialization)]

#[macro_use]
extern crate pyo3;

use pyo3::prelude::*;

#[pyclass(module = "unit_converter")]
struct Temperature {
    celsius: f32,
}

#[pymethods]
impl Temperature {
    #[new]
    fn new(obj: &PyRawObject, temperature: f32) {
        obj.init(Temperature {
            celsius: temperature,
        });
    }

    fn to_fahrenheit(&self) -> f32 {
        self.celsius * 1.8 + 32.0
    }

    fn windchill(&self, wind_speed_kph: f32) -> f32 {
        // From https://www.weather.gov/media/epz/wxcalc/windChill.pdf
        // And https://www.calcunation.com/calculator/wind-chill-celsius.php
        13.12 + (0.6215 * self.celsius) - (11.37 * wind_speed_kph.powf(0.16))
            + (0.3965 * self.celsius * wind_speed_kph.powf(0.16))
    }
}

// Refer to: https://www.weather.gov/epz/wxcalc_windchill
// TODO: Add a test that -20 C and 32 kph feels like -32.9

#[pyfunction]
fn batch_converter(temperatures: Vec<f32>) -> Vec<f32> {
    temperatures
        .iter()
        .map(|temperature| {
            let temp = Temperature {
                celsius: temperature.clone(),
            };
            temp.to_fahrenheit()
        })
        .collect()
}

#[pymodule]
fn unit_converter(_py: Python, module: &PyModule) -> PyResult<()> {
    module.add_class::<Temperature>()?;
    module.add_wrapped(wrap_pyfunction!(batch_converter))?;
    Ok(())
}

#[cfg(test)]
mod tests {

    use crate::Temperature;

    #[test]
    fn conversion_celsius_to_fahrenheit() {
        let temperature = Temperature { celsius: 25.0 };
        assert_eq!(temperature.to_fahrenheit(), 77.0);
    }
}
